<?php

namespace AnalyticsWP\Lib;

/**
 * @psalm-import-type DateRangeValue from DateRange
 * 
 * @psalm-type Device = value-of<APIServer::DEVICES>
 * @psalm-type TopStatName = value-of<APIServer::TOP_STAT_NAMES>
 *
 * @psalm-type TopStat = array{
 *   name: TopStatName,
 *   value: int|float,
 *   change: int|float,
 *   comparison_value: int|float
 * }
 * 
 * @psalm-type TopStats = list<TopStat>
 *
 * @psalm-type ChartDataPoint = array{x: string, y: int|float}
 * 
 * @psalm-type ChartData = array{
 *   "Unique people": ChartDataPoint[],
 *   "Total pageviews": ChartDataPoint[],
 *   "Views per person": ChartDataPoint[],
 *   "Window shoppers rate": ChartDataPoint[],
 * }
 *
 * @psalm-type TopSource = array{
 *   source: string,
 *   visits: int,
 *   bounceRate: int,
 * }
 *
 * @psalm-type TopPage = array{
 *   pageUrl: string,
 *   visits: int,
 * }
 *
 * @psalm-type TopCountry = array{
 *   country: string,
 *   visits: int
 * }
 *
 * @psalm-type TopDevice = array{
 *   device: Device,
 *   visits: int
 * }
 *
 * @psalm-type MainGraphData = array{
 *   currently_selected_top_stat_name: TopStatName,
 *   top_stats: TopStat[],
 *   chart_data: ChartData
 * }
 * 
 * @psalm-type DashboardFilters = array{
 *   date_range: DateRangeValue,
 *   custom_date_range: array{start_date: string, end_date: string},
 *   custom_sql: string
 * }
 * 
 * @psalm-type SavedFilter = array{
 *   id: string,
 *   name: string,
 *   description: string,
 *   created_at: string,
 *   created_by_user_id: int,
 *   filters: DashboardFilters
 * }
 *
 * @psalm-type StatsDashboardData = array{
 *   is_loading: bool,
 *   filters: DashboardFilters,
 *   main_graph: MainGraphData,
 *   top_sources: TopSource[],
 *   top_pages: TopPage[],
 *   top_countries: TopCountry[],
 *   top_devices: TopDevice[],
 *   debug: array,
 *   saved_filters: SavedFilter[]
 * }
 */
class APIServer
{
    const CACHE_DURATION_IN_SECONDS = 300;

    const TOP_STAT_NAMES = [
        "Unique people",
        "Total pageviews",
        "Views per person",
        "Window shoppers rate"
    ];

    const DEVICES = [
        "tablet",
        "mobile",
        "desktop",
        "unknown"
    ];

    public static function init(): void
    {
        add_action('wp_ajax_analyticswp_get_stats', [self::class, 'handle_get_stats']);
        add_action('wp_ajax_nopriv_analyticswp_get_stats', [self::class, 'handle_get_stats']);

        add_action('wp_ajax_analyticswp_get_realtime_stats', [self::class, 'handle_get_realtime_stats']);
        add_action('wp_ajax_nopriv_analyticswp_get_realtime_stats', [self::class, 'handle_get_realtime_stats']);

        add_action('wp_ajax_analyticswp_get_live_events', [self::class, 'handle_get_live_events']);
        add_action('wp_ajax_nopriv_analyticswp_get_live_events', [self::class, 'handle_get_live_events']);

        add_action('wp_ajax_analyticswp_save_filter', [SavedFilters::class, 'handle_ajax_save_filter']);
        add_action('wp_ajax_nopriv_analyticswp_save_filter', [SavedFilters::class, 'handle_ajax_save_filter']);

        add_action('wp_ajax_analyticswp_delete_filter', [SavedFilters::class, 'handle_ajax_delete_filter']);
        add_action('wp_ajax_nopriv_analyticswp_delete_filter', [SavedFilters::class, 'handle_ajax_delete_filter']);

        add_action('wp_ajax_analyticswp_get_integrations', [self::class, 'handle_ajax_get_integrations']);
        add_action('wp_ajax_nopriv_analyticswp_get_integrations', [self::class, 'handle_ajax_get_integrations']);

        add_action('wp_ajax_analyticswp_update_integration', [self::class, 'handle_ajax_update_integration']);
        add_action('wp_ajax_nopriv_analyticswp_update_integration', [self::class, 'handle_ajax_update_integration']);
    }



    public static function handle_get_stats(): void
    {
        // Set CORS headers
        header("Access-Control-Allow-Origin: *"); // Allows all domains. For specific domains, replace '*' with the domain.
        header("Access-Control-Allow-Methods: GET, POST, OPTIONS");
        header("Access-Control-Allow-Headers: Content-Type, X-Requested-With");
        header("Content-Type: application/json");

        // check that the user is a logged in administrator (has the manage_options capability)
        // TODO figure this out. It doesn't work in development mode for some reason...but maybe works in production mode (?)
        // if (!current_user_can('manage_options')) {
        //     wp_send_json_error(array('message' => 'You do not have permission to access this resource.'));
        // }

        // Handle pre-flight requests
        if (isset($_SERVER['REQUEST_METHOD']) && ($_SERVER['REQUEST_METHOD'] === 'OPTIONS')) {
            exit(0);
        }

        // TODO Check nonce for security
        // check_ajax_referer('analyticswp_get_stats', 'nonce');

        $date_range = DateRange::fromPost($_POST);

        if (!($date_range instanceof DateRange)) {
            // If it's a string, it's an error message
            wp_send_json_error(array('message' => $date_range));
        }

        // customSQL (from filter builder)
        $custom_sql = Validators::str(isset($_POST['custom_sql']) ? wp_unslash($_POST['custom_sql']) : '');



        // TODO - This is where i left off today.
        if (Event::count() <= 10000000) { // Don't cache if the db is small.
            $stats_dashboard_data = APIServerRealData::generateStatsDashboardData($date_range, $custom_sql);
        } else {
            /** @psalm-suppress PossiblyInvalidArgument - This one is ok. */
            $stats_dashboard_data = Utils::cache_function_result([APIServerRealData::class, 'generateStatsDashboardData'], [$date_range, $custom_sql], self::CACHE_DURATION_IN_SECONDS);
        }

        $current_user = wp_get_current_user();
        if ($current_user->ID === 0) {
            $stats_dashboard_data['admin_username'] = 'there';
        } else {
            $stats_dashboard_data['admin_username'] = $current_user->user_login;
        }

        $stats_dashboard_data['analyticswp_version'] = ANALYTICSWP_VERSION;
        // $stats_dashboard_data['admin_username'] = $current_user->user_login ?? 'there';

        // return the response
        wp_send_json_success($stats_dashboard_data);
    }

    public static function handle_get_realtime_stats(): void
    {
        // Set CORS headers
        header("Access-Control-Allow-Origin: *"); // Allows all domains. For specific domains, replace '*' with the domain.
        header("Access-Control-Allow-Methods: GET, POST, OPTIONS");
        header("Access-Control-Allow-Headers: Content-Type, X-Requested-With");
        header("Content-Type: application/json");

        // Handle pre-flight requests
        if (isset($_SERVER['REQUEST_METHOD']) && ($_SERVER['REQUEST_METHOD'] === 'OPTIONS')) {
            exit(0);
        }

        // TODO Check nonce for security
        // check_ajax_referer('analyticswp_nonce', 'nonce');

        $realtime_data = APIServerRealData::generateRealtimeStatsDashboardData();

        // return the response
        wp_send_json_success($realtime_data);
    }


    public static function handle_get_live_events(): void
    {
        // Set CORS headers
        header("Access-Control-Allow-Origin: *"); // Allows all domains. For specific domains, replace '*' with the domain.
        header("Access-Control-Allow-Methods: GET, POST, OPTIONS");
        header("Access-Control-Allow-Headers: Content-Type, X-Requested-With");
        header("Content-Type: application/json");

        // Handle pre-flight requests
        if (isset($_SERVER['REQUEST_METHOD']) && ($_SERVER['REQUEST_METHOD'] === 'OPTIONS')) {
            exit(0);
        }

        // get start_time and end_time from the request
        $start_time = isset($_POST['start_time']) ? ($_POST['start_time']) : null;
        $end_time = isset($_POST['end_time']) ? ($_POST['end_time']) : null;

        // TODO Check nonce for security
        // check_ajax_referer('analyticswp_nonce', 'nonce');

        // $realtime_data = APIServerRealData::generateRealtimeStatsDashboardData();

        $data = Event::getBetweenTimes($start_time, $end_time);

        // return the response
        wp_send_json_success($data);
    }


    //////////////////////////////////////////////////////////////////////
    //////////////////////////////////////////////////////////////////////
    public static function handle_ajax_get_integrations(): void
    {
        // Set CORS headers
        header("Access-Control-Allow-Origin: *"); // Allows all domains. For specific domains, replace '*' with the domain.
        header("Access-Control-Allow-Methods: GET, POST, OPTIONS");
        header("Access-Control-Allow-Headers: Content-Type, X-Requested-With");
        header("Content-Type: application/json");

        // Handle pre-flight requests
        if (isset($_SERVER['REQUEST_METHOD']) && ($_SERVER['REQUEST_METHOD'] === 'OPTIONS')) {
            exit(0);
        }

        $integrations = Integrations::get_integrations();

        // return the response
        wp_send_json_success($integrations);
    }

    public static function handle_ajax_update_integration(): void
    {
        // Set CORS headers
        header("Access-Control-Allow-Origin: *"); // Allows all domains. For specific domains, replace '*' with the domain.
        header("Access-Control-Allow-Methods: GET, POST, OPTIONS");
        header("Access-Control-Allow-Headers: Content-Type, X-Requested-With");
        header("Content-Type: application/json");

        // Handle pre-flight requests
        if (isset($_SERVER['REQUEST_METHOD']) && ($_SERVER['REQUEST_METHOD'] === 'OPTIONS')) {
            exit(0);
        }

        $integration_data = isset($_POST['integration_data']) ?  json_decode(stripslashes($_POST['integration_data']), true) : null;

        if (!is_array($integration_data)) {
            wp_send_json_error(array('message' => 'Invalid integration data.'));
        }

        $integration_slug = Validators::str($integration_data['slug']);
        $integration_enabled = (bool)($integration_data['enabled']);

        $integrations = Integrations::update_integration_is_enabled($integration_slug, $integration_enabled);

        // return the response
        wp_send_json_success($integrations);
    }
}
